@@ -364,8 +364,7 @@ pub fn expand_node(circuit: CircuitRc, inputs: &Vec<CircuitRc>) -> Result<Circui
                         Ok((
                             ModuleArgSpec {
                                 symbol,
-                                batchable: arg_spec.batchable,
-                                expandable: arg_spec.expandable,
+                                ..arg_spec.clone()
                             },
                             inp.clone(),
                         ))
@@ -481,6 +480,17 @@ pub enum ExpandError {
         arg_spec: ModuleArgSpec,
         spec_circuit: CircuitRc,
     },
+    #[error("new_size={fancy_new_size} != old_size={old_size} and old_size not symbolic at dim={dim}\n{}\n({e_name})",
+        format!("node_shape={:?}, arg_spec={:?} spec_circuit={:?}", node_shape, arg_spec, spec_circuit),
+        fancy_new_size=SymbolicSizeProduct::from(*new_size))]
+    ModuleTriedToExpandOnNonSymbolicSizeAndBanned {
+        new_size: usize,
+        old_size: usize,
+        dim: usize,
+        node_shape: Shape,
+        arg_spec: ModuleArgSpec,
+        spec_circuit: CircuitRc,
+    },
 }
 
 #[pyfunction]
