@@ -5,6 +5,7 @@ use macro_rules_attribute::apply;
 use pyo3::prelude::*;
 use rr_util::{
     pycall,
+    symbolic_size::SymbolicSizeProduct,
     tensor_util::{Shape, TorchDeviceDtype},
     util::{counts_g_1, is_unique},
 };
@@ -72,6 +73,25 @@ impl ModuleSpec {
                     spec_circuit: self.circuit.clone()
                 });
             }
+            if arg_spec.ban_non_symbolic_size_expand {
+                for (dim, (&new_size, &old_size)) in node.shape()
+                    [node.rank() - arg_spec.symbol.rank()..]
+                    .iter()
+                    .zip(arg_spec.symbol.shape())
+                    .enumerate()
+                {
+                    if new_size != old_size && !SymbolicSizeProduct::has_symbolic(old_size) {
+                        bail!(ExpandError::ModuleTriedToExpandOnNonSymbolicSizeAndBanned {
+                            old_size,
+                            new_size,
+                            dim,
+                            node_shape: node.shape().clone(),
+                            arg_spec: arg_spec.clone(),
+                            spec_circuit: self.circuit.clone()
+                        });
+                    }
+                }
+            }
         }
         replace_expand_bottom_up(self.circuit.clone(), |c| {
             self.arg_specs
@@ -282,7 +302,7 @@ impl ModuleSpec {
     pub fn new_all_free(circuit: CircuitRc, check_unique_arg_names: bool) -> Result<Self> {
         let arg_specs = all_free(circuit.clone())
             .into_iter()
-            .map(|x| ModuleArgSpec::new(x, true, true))
+            .map(|x| ModuleArgSpec::new(x, true, true, true))
             .collect();
         let out =
             Self::new(circuit, arg_specs, true, false).expect("method should guarantee valid");
@@ -364,8 +384,7 @@ impl ModuleSpec {
         let arg_specs: Vec<ModuleArgSpec> = zip(&self.arg_specs, shapes)
             .map(|(arg_spec, shape)| ModuleArgSpec {
                 symbol: Symbol::new(shape, arg_spec.symbol.uuid, arg_spec.symbol.name_cloned()),
-                batchable: arg_spec.batchable,
-                expandable: arg_spec.expandable,
+                ..arg_spec.clone()
             })
             .collect();
 
@@ -387,12 +406,14 @@ impl ModuleSpec {
 #[pyclass]
 #[derive(Debug, Clone, Hash, PartialEq, Eq)]
 pub struct ModuleArgSpec {
-    #[pyo3(get)]
+    #[pyo3(get, set)]
     pub symbol: Symbol,
-    #[pyo3(get)]
+    #[pyo3(get, set)]
     pub batchable: bool,
-    #[pyo3(get)]
+    #[pyo3(get, set)]
     pub expandable: bool,
+    #[pyo3(get, set)]
+    pub ban_non_symbolic_size_expand: bool,
 }
 
 impl Default for ModuleArgSpec {
@@ -401,6 +422,7 @@ impl Default for ModuleArgSpec {
             symbol: Symbol::new_with_none_uuid([].into_iter().collect(), None),
             batchable: true,
             expandable: true,
+            ban_non_symbolic_size_expand: true,
         }
     }
 }
@@ -410,21 +432,34 @@ impl ModuleArgSpec {
     #[new]
     #[args(
         batchable = "Self::default().batchable",
-        expandable = "Self::default().expandable"
+        expandable = "Self::default().expandable",
+        ban_non_symbolic_size_expand = "Self::default().ban_non_symbolic_size_expand"
     )]
-    fn new(symbol: Symbol, batchable: bool, expandable: bool) -> Self {
+    fn new(
+        symbol: Symbol,
+        batchable: bool,
+        expandable: bool,
+        ban_non_symbolic_size_expand: bool,
+    ) -> Self {
         Self {
             symbol,
             batchable,
             expandable,
+            ban_non_symbolic_size_expand,
         }
     }
     #[staticmethod]
     #[args(
         batchable = "Self::default().batchable",
-        expandable = "Self::default().expandable"
+        expandable = "Self::default().expandable",
+        ban_non_symbolic_size_expand = "false", // I think this is right default for this?
     )]
-    pub fn just_name_shape(circuit: CircuitRc, batchable: bool, expandable: bool) -> Self {
+    pub fn just_name_shape(
+        circuit: CircuitRc,
+        batchable: bool,
+        expandable: bool,
+        ban_non_symbolic_size_expand: bool,
+    ) -> Self {
         Self {
             symbol: Symbol::new_with_random_uuid(
                 circuit.info().shape.clone(),
@@ -432,6 +467,7 @@ impl ModuleArgSpec {
             ),
             batchable,
             expandable,
+            ban_non_symbolic_size_expand,
         }
     }
 
