@@ -1,14 +1,12 @@
-use std::{fmt::Debug, ops::Deref, sync::Arc};
+use std::{collections::BTreeSet, fmt::Debug, ops::Deref, sync::Arc};
 
 use anyhow::Result;
-use macro_rules_attribute::apply;
-use pyo3::{exceptions::PyTypeError, prelude::*, pyclass::CompareOp, types::PyDict, PyObject};
+use pyo3::{prelude::*, pyclass::CompareOp, types::PyDict, PyObject};
 use rr_util::{
     atr,
     py_types::{use_rust_comp, SELF_MODULE},
-    pycall, python_error_exception,
+    pycall,
 };
-use thiserror::Error;
 
 use crate::CircuitRc;
 
@@ -20,18 +18,11 @@ pub enum OpaqueIterativeMatcherVal {
 
 impl<'source> FromPyObject<'source> for OpaqueIterativeMatcherVal {
     fn extract(inp: &'source PyAny) -> PyResult<Self> {
-        let ty = inp.get_type();
-        let name = ty.name()?;
-        let expected_name = "IterativeMatcher";
-        if name != expected_name {
-            return Err(anyhow::Error::from(IncorrectOpaqueTypeError::NameMismatch {
-                actual_name: name.to_owned(),
-                expected_name: expected_name.to_owned(),
-            })
-            .into());
-        }
-
-        Ok(Self::Py(inp.into()))
+        Python::with_gil(|py| {
+            Ok(Self::Py(
+                atr!(SELF_MODULE, IterativeMatcher, raw).call1(py, (inp,))?,
+            ))
+        })
     }
 }
 
@@ -228,6 +219,8 @@ pub struct EndDepthOpaqueIterativeMatcher {
 // why bother impl-ing if we also need to have python conversion?
 // We do this so that this so depth matcher can be used in rust without assuming python library is installed!
 // (same for below with Never matcher)
+//
+// This is a bit gross and sad...
 impl ToPyObject for EndDepthOpaqueIterativeMatcher {
     fn to_object(&self, _py: Python<'_>) -> PyObject {
         assert_eq!(
@@ -285,6 +278,29 @@ impl OpaqueIterativeMatcher for NeverOpaqueIterativeMatcher {
     }
 }
 
+macro_rules! defer_to_py {
+    (
+    $(
+        fn $fn_name:ident(
+            &self,
+            $($arg:ident : $arg_ty:ty),* $(,)?
+        ) -> $ret_ty:ty;
+    )*
+    ) => (
+    $(
+        pub(super) fn $fn_name(
+            &self,
+            $($arg : $arg_ty),*
+        ) -> $ret_ty {
+            Python::with_gil(|py| {
+                atr!(self.to_object(py), $fn_name, raw).call1(py, ($($arg,)*)).map(|x| x.extract(py).unwrap())
+            })
+        }
+    )*
+
+    )
+}
+
 impl OpaqueIterativeMatcherVal {
     pub fn noop_traversal_raw() -> PyObject {
         atr!(SELF_MODULE, traversal, raw)
@@ -308,16 +324,28 @@ impl OpaqueIterativeMatcherVal {
                 .unwrap_or_else(|| Self::noop_traversal_raw())
         })
     }
-}
 
-#[apply(python_error_exception)]
-#[base_error_name(IncorrectOpaqueType)]
-#[base_exception(PyTypeError)]
-#[derive(Error, Debug, Clone)]
-pub enum IncorrectOpaqueTypeError {
-    #[error("expected={expected_name} actual={actual_name} ({e_name})")]
-    NameMismatch {
-        actual_name: String,
-        expected_name: String,
-    },
+    // below methods should just be used for PyCircuitBase
+    defer_to_py!(
+        fn update(
+            &self,
+            circuit: CircuitRc,
+            transform: PyObject,
+            cache_transform: bool,
+            cache_update: bool,
+            fancy_validate: bool,
+        ) -> PyResult<CircuitRc>;
+
+        fn get(&self, circuit: CircuitRc, fancy_validate: bool) -> PyResult<BTreeSet<CircuitRc>>;
+
+        fn get_unique_op(
+            &self,
+            circuit: CircuitRc,
+            fancy_validate: bool,
+        ) -> PyResult<Option<CircuitRc>>;
+
+        fn get_unique(&self, circuit: CircuitRc, fancy_validate: bool) -> PyResult<CircuitRc>;
+
+        fn are_any_found(&self, circuit: CircuitRc) -> PyResult<bool>;
+    );
 }
