@@ -25,6 +25,8 @@ use crate::{
     Rearrange, Scalar, Scatter, SetSymbolicShape, StoredCumulantVar, Symbol, Tag,
 };
 
+const FANCY_PREFIX: &str = "fancy:"; // Should only contain ascii characters
+
 #[pyclass]
 #[derive(Clone, Debug)]
 pub struct Parser {
@@ -592,7 +594,18 @@ impl Parser {
                 ps.name.clone(),
             )?
             .rc(),
-            T::Einsum => Einsum::from_einsum_string(&ps.extra, children, ps.name.clone())?.rc(),
+            T::Einsum => if ps.extra.starts_with(FANCY_PREFIX) {
+                Einsum::from_fancy_string(
+                    &ps.extra[FANCY_PREFIX.len()..],
+                    children,
+                    ps.name.clone(),
+                )
+            } else {
+                Einsum::from_einsum_string(&ps.extra, children.clone(), ps.name.clone()).context(
+                    "Couldn't parse einsum string. If you wanted to parse a fancy einsum string, prefix it by \"fancy:\".",
+                )
+            }?
+            .rc(),
             T::Rearrange => {
                 expected_k_children(1)?;
                 Rearrange::from_string(children[0].clone(), &ps.extra, ps.name.clone())?.rc()
@@ -659,15 +672,15 @@ impl Parser {
                         })
                     }
                     (0..((children.len() - 1) / 2))
-                        .map(|_| [true, true])
+                        .map(|_| [true, true, true])
                         .collect()
                 } else {
                     ps.extra
                         .split(",")
                         .map(|x| {
                             let num_chars = x.chars().count();
-                            if num_chars != 2 {
-                                bail!(ParseCircuitError::Expected2CharsPerForModuleExtra {
+                            if num_chars != 3 {
+                                bail!(ParseCircuitError::Expected3CharsPerForModuleExtra {
                                     name: ps.name.clone(),
                                     num_chars,
                                     extra: ps.extra.clone(),
@@ -681,7 +694,7 @@ impl Parser {
                                 .unwrap();
                             Ok(out)
                         })
-                        .collect::<Result<Vec<[bool; 2]>>>()?
+                        .collect::<Result<Vec<[bool; 3]>>>()?
                 };
 
                 if children.len() != 2 * arg_info.len() + 1 {
@@ -699,29 +712,32 @@ impl Parser {
                 let (arg_specs, nodes) = rest
                     .chunks_exact(2)
                     .zip(arg_info)
-                    .map(|(sym_inp, [batchable, expandable])| {
-                        let (sym, inp) = if let [sym, inp] = sym_inp {
-                            (sym, inp)
-                        } else {
-                            unreachable!()
-                        };
-                        let symbol = sym
-                            .as_symbol()
-                            .ok_or_else(|| ConstructError::ModuleExpectedSymbol {
-                                actual_circuit: sym.clone(),
-                            })
-                            .context("module expected symbol in parse")?
-                            .clone();
+                    .map(
+                        |(sym_inp, [batchable, expandable, ban_non_symbolic_size_expand])| {
+                            let (sym, inp) = if let [sym, inp] = sym_inp {
+                                (sym, inp)
+                            } else {
+                                unreachable!()
+                            };
+                            let symbol = sym
+                                .as_symbol()
+                                .ok_or_else(|| ConstructError::ModuleExpectedSymbol {
+                                    actual_circuit: sym.clone(),
+                                })
+                                .context("module expected symbol in parse")?
+                                .clone();
 
-                        Ok((
-                            ModuleArgSpec {
-                                symbol,
-                                batchable,
-                                expandable,
-                            },
-                            inp.clone(),
-                        ))
-                    })
+                            Ok((
+                                ModuleArgSpec {
+                                    symbol,
+                                    batchable,
+                                    expandable,
+                                    ban_non_symbolic_size_expand,
+                                },
+                                inp.clone(),
+                            ))
+                        },
+                    )
                     .collect::<Result<Vec<_>>>()?
                     .into_iter()
                     .unzip();
@@ -928,8 +944,8 @@ pub enum ParseCircuitError {
         err_strs: String,
     },
 
-    #[error("num_chars={num_chars} extra='{extra}' name={name:?}\nmodule extra should look like 'tt,tf,ft' and we check for 2 chars per comma delimited item.\n({e_name})")]
-    Expected2CharsPerForModuleExtra {
+    #[error("num_chars={num_chars}!=3 extra='{extra}' name={name:?}\nmodule extra should look like 'ttt,tft,ftf' and we check for 3 chars per comma delimited item.\n({e_name})")]
+    Expected3CharsPerForModuleExtra {
         name: Option<String>,
         num_chars: usize,
         extra: String,
@@ -938,7 +954,7 @@ pub enum ParseCircuitError {
     #[error("num_children={num_children} != 2 * num_args={num_args:?} + 1\nname={name:?} extra={extra}\n{}\n{}\n{}\n({e_name})",
     "modules have 1 child for the spec and then a symbol and a input circuit for each arg (2 * num_args + 1)",
     "this implies the number of children is always odd",
-    "num_args is based on parsing the batchable and expandable info from 'extra' (see above) (in a form like 'tt,tf,ft' (Some(3)) or 'all_t' (None))")]
+    "num_args is based on parsing the batchable and expandable info from 'extra' (see above) (in a form like 'ttt,tft,ftf' (Some(3)) or 'all_t' (None))")]
     ModuleInconsistentNumChildren {
         name: Option<String>,
         num_children: usize,
