@@ -1,18 +1,19 @@
 use std::{
-    collections::HashSet,
     fmt,
-    fmt::Debug,
+    fmt::{Debug, Write},
     hash::{Hash, Hasher},
     sync::Mutex,
 };
 
 use anyhow::{anyhow, bail, Context, Error, Result};
+use itertools::Itertools;
 use num_bigint::BigUint;
 use pyo3::{once_cell::GILLazy, prelude::*};
 use rr_util::{
     fn_struct, py_types::MaybeNotSet, python_println, symbolic_size::SymbolicSizeProduct,
+    util::ALPHABET,
 };
-use rustc_hash::FxHashMap as HashMap;
+use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
 
 use crate::{
     circuit_utils::{count_nodes, total_flops},
@@ -24,7 +25,16 @@ use crate::{
 };
 
 // TODO: improve color scheme
-const COLOR_CODES: [usize; 14] = [31, 32, 33, 34, 35, 36, 90, 91, 92, 93, 94, 95, 96, 97];
+const COLOR_CODES: [usize; 14] = [
+    31, // Red
+    32, // Green
+    33, // Yellow
+    34, // Blue
+    35, // Magenta
+    36, // Cyan
+    90, // White
+    91, 92, 93, 94, 95, 96, 97,
+];
 
 const BAR: &'static str = "│";
 const TEE: &'static str = "├";
@@ -33,6 +43,7 @@ const UP_ELBOW: &'static str = "└";
 const DOWN_ELBOW: &'static str = "┌";
 
 fn_struct!(pub CircuitToColorCode:Fn(circuit:CircuitRc)->usize);
+fn_struct!(pub CircuitCommenter:Fn(circuit:CircuitRc)->String);
 
 #[pyclass]
 #[derive(Clone)]
@@ -52,6 +63,10 @@ pub struct PrintOptions {
     pub force_use_serial_numbers: bool,
     #[pyo3(get, set)]
     pub colorer: Option<CircuitToColorCode>,
+    #[pyo3(get, set)]
+    pub comment_arg_names: bool,
+    #[pyo3(get, set)]
+    pub commenters: Vec<CircuitCommenter>,
 }
 
 impl Default for PrintOptions {
@@ -65,6 +80,8 @@ impl Default for PrintOptions {
             print_size_threshold: None,
             force_use_serial_numbers: false,
             colorer: None,
+            comment_arg_names: false,
+            commenters: vec![],
         }
         .validate_ret()
         .unwrap()
@@ -162,7 +179,9 @@ impl PrintOptions {
         arrows = "PrintOptions::default().arrows",
         print_size_threshold = "None",
         force_use_serial_numbers = "PrintOptions::default().force_use_serial_numbers",
-        colorer = "None"
+        colorer = "None",
+        comment_arg_names = "PrintOptions::default().comment_arg_names",
+        commenters = "PrintOptions::default().commenters"
     )]
     pub fn new(
         bijection: bool,
@@ -173,6 +192,8 @@ impl PrintOptions {
         print_size_threshold: Option<usize>,
         force_use_serial_numbers: bool,
         colorer: Option<CircuitToColorCode>,
+        comment_arg_names: bool,
+        commenters: Vec<CircuitCommenter>,
     ) -> Result<Self> {
         let result = Self {
             bijection,
@@ -183,6 +204,8 @@ impl PrintOptions {
             print_size_threshold,
             force_use_serial_numbers,
             colorer,
+            comment_arg_names,
+            commenters,
         };
         result.validate()?;
         Ok(result)
@@ -196,7 +219,9 @@ impl PrintOptions {
         arrows = "Default::default()",
         print_size_threshold = "Default::default()",
         force_use_serial_numbers = "Default::default()",
-        colorer = "Default::default()"
+        colorer = "Default::default()",
+        comment_arg_names = "Default::default()",
+        commenters = "Default::default()"
     )]
     pub fn evolve(
         &self,
@@ -208,6 +233,8 @@ impl PrintOptions {
         print_size_threshold: MaybeNotSet<Option<usize>>,
         force_use_serial_numbers: MaybeNotSet<bool>,
         colorer: MaybeNotSet<Option<CircuitToColorCode>>,
+        comment_arg_names: MaybeNotSet<bool>,
+        commenters: MaybeNotSet<Vec<CircuitCommenter>>,
     ) -> Result<Self> {
         let cloned = self.clone();
         Self::new(
@@ -225,6 +252,8 @@ impl PrintOptions {
                 .0
                 .unwrap_or(cloned.force_use_serial_numbers),
             colorer.0.unwrap_or(cloned.colorer),
+            comment_arg_names.0.unwrap_or(cloned.comment_arg_names),
+            commenters.0.unwrap_or(cloned.commenters),
         )
     }
 
@@ -364,7 +393,7 @@ impl PrintOptions {
                             .spec
                             .arg_specs
                             .iter()
-                            .all(|x| x.batchable && x.expandable)
+                            .all(|x| x.batchable && x.expandable && x.ban_non_symbolic_size_expand)
                     {
                         "all_t".to_owned()
                     } else {
@@ -375,9 +404,10 @@ impl PrintOptions {
                             .iter()
                             .map(|arg_spec| {
                                 format!(
-                                    "{}{}",
+                                    "{}{}{}",
                                     TerseBool(arg_spec.batchable),
-                                    TerseBool(arg_spec.expandable)
+                                    TerseBool(arg_spec.expandable),
+                                    TerseBool(arg_spec.ban_non_symbolic_size_expand)
                                 )
                             })
                             .collect::<Vec<_>>()
@@ -474,27 +504,16 @@ impl PrintOptions {
             is_last_child: &Vec<bool>,
             traversal: Option<OpaqueIterativeMatcherVal>,
             disallow_name_ident: Option<&HashMap<String, bool>>,
+            node_comments: Vec<String>,
         ) -> Result<()> {
-            if selfy.arrows {
-                if depth > 1 {
-                    for i in 0..(depth - 1) {
-                        result.push_str(if is_last_child[i] { " " } else { BAR });
-                        result.push(' ');
-                    }
-                }
-                if depth > 0 {
-                    result.push_str(if *is_last_child.last().unwrap() {
-                        UP_ELBOW
-                    } else {
-                        TEE
-                    });
-                    result.push_str(ARROW);
-                }
-            } else {
-                result.push_str(&" ".repeat(depth * 2));
-            }
+            result.push_str(&last_child_arrows(is_last_child, true, selfy.arrows));
             if let Some(prev) = seen_hashes.get(&circ.info().hash) {
                 result.push_str(prev);
+                for comment in node_comments {
+                    if !comment.is_empty() {
+                        write!(result, " # {}", comment).unwrap();
+                    }
+                }
                 result.push('\n');
                 return Ok(());
             }
@@ -562,9 +581,53 @@ impl PrintOptions {
                     result.push('\n');
                     return Ok(());
                 };
-
+            for comment in node_comments.into_iter().chain(
+                selfy
+                    .commenters
+                    .iter()
+                    .map(|commenter| commenter.call(circ.clone().rc()))
+                    .collect::<Result<Vec<String>>>()?,
+            ) {
+                if !comment.is_empty() {
+                    write!(result, " # {}", comment).unwrap();
+                }
+            }
             result.push('\n');
             for (i, (child, new_traversal)) in circ.children().zip(new_traversal_per).enumerate() {
+                let new_last_child = is_last_child
+                    .iter()
+                    .copied()
+                    .chain(std::iter::once(i == num_children - 1))
+                    .collect();
+                let child_specific_commenters = if selfy.comment_arg_names {
+                    vec![match circ {
+                        Circuit::Einsum(ein) => ein.args[i]
+                            .1
+                            .iter()
+                            .map(|int| ALPHABET[*int as usize].clone())
+                            .join(""),
+                        Circuit::DiscreteVar(_dv) => match i {
+                            0 => "Values".to_owned(),
+                            1 => "Probs and Group".to_owned(),
+                            _ => {
+                                panic!()
+                            }
+                        },
+                        Circuit::Conv(_conv) => match i {
+                            0 => "Input".to_owned(),
+                            1 => "Filter".to_owned(),
+                            _ => {
+                                panic!()
+                            }
+                        },
+                        Circuit::StoredCumulantVar(scv) => {
+                            format!("K{}", scv.cumulants.keys().nth(i).unwrap())
+                        }
+                        _ => "".to_owned(),
+                    }]
+                } else {
+                    vec![]
+                };
                 recurse(
                     &child,
                     depth + 1,
@@ -572,13 +635,10 @@ impl PrintOptions {
                     seen_hashes,
                     runnning_serial_number,
                     selfy,
-                    &is_last_child
-                        .iter()
-                        .copied()
-                        .chain(std::iter::once(i == num_children - 1))
-                        .collect(),
+                    &new_last_child,
                     new_traversal,
                     disallow_name_ident,
+                    child_specific_commenters,
                 )?;
             }
             Ok(())
@@ -596,6 +656,7 @@ impl PrintOptions {
                 &vec![],
                 self.traversal.clone(),
                 disallow_name_ident.as_ref(),
+                vec![],
             )?;
         }
 
@@ -760,3 +821,28 @@ pub fn color(string: &str, color_int: usize) -> String {
         string
     )
 }
+
+pub fn last_child_arrows(last_child: &Vec<bool>, is_output: bool, arrows: bool) -> String {
+    if !arrows {
+        return "  ".repeat(last_child.len());
+    }
+    let depth = last_child.len();
+    let mut result = "".to_owned();
+    for i in 0..depth.saturating_sub(1) {
+        result.push_str(if last_child[i] { " " } else { BAR });
+        result.push(' ');
+    }
+    if depth > 0 {
+        if is_output {
+            result.push_str(if *last_child.last().unwrap() {
+                UP_ELBOW
+            } else {
+                TEE
+            });
+            result.push_str(ARROW);
+        } else {
+            write!(result, "{} ", BAR).unwrap();
+        }
+    }
+    result
+}
