@@ -1,5 +1,4 @@
 use std::{
-    collections::HashSet,
     fmt::Debug,
     hash::{Hash, Hasher},
 };
@@ -8,7 +7,7 @@ use anyhow::{anyhow, bail, Context, Result};
 use pyo3::prelude::*;
 use rand::Rng;
 use rr_util::{fn_struct, py_types::MaybeNotSet, simple_default, util::hash_to_string};
-use rustc_hash::FxHashMap as HashMap;
+use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
 
 use crate::{
     opaque_iterative_matcher::OpaqueIterativeMatcherVal,
