@@ -56,8 +56,8 @@ pub fn batch_to_concat(circuit: CircuitRc, axis: usize, num_batches: usize) -> R
                         Symbol::new_with_random_uuid(sub.info().shape.clone(), None),
                         BTreeMap::from([(*i as u8, name.clone())]),
                     ),
-                    batchable: true,
-                    expandable: true,
+                    ban_non_symbolic_size_expand: false, // I think this needs to be false?
+                    ..Default::default()
                 },
             )
         })
@@ -70,8 +70,8 @@ pub fn batch_to_concat(circuit: CircuitRc, axis: usize, num_batches: usize) -> R
                 sub.clone(),
                 ModuleArgSpec {
                     symbol: Symbol::new_with_random_uuid(sub.info().shape.clone(), None),
-                    batchable: true,
-                    expandable: true,
+                    ban_non_symbolic_size_expand: false, // I think this needs to be false?
+                    ..Default::default()
                 },
             )
         }))
