@@ -1,6 +1,6 @@
 use circuit_base::{
     circuit_utils::{replace_nodes, toposort_circuit},
-    evaluate_fn, ArrayConstant, Circuit, CircuitNode, CircuitRc,
+    evaluate, Array, Circuit, CircuitNode, CircuitRc,
 };
 use num_bigint::BigUint;
 use pyo3::prelude::*;
@@ -56,8 +56,8 @@ pub fn is_close(a: Tensor, b: Tensor) -> Result<PyObject, PyErr> {
 }
 
 pub fn assert_is_close(a: CircuitRc, b: CircuitRc) {
-    let a_ten = evaluate_fn(a.clone()).unwrap();
-    let b_ten = evaluate_fn(b.clone()).unwrap();
+    let a_ten = evaluate(a.clone()).unwrap();
+    let b_ten = evaluate(b.clone()).unwrap();
     let close_result = is_close(a_ten.clone(), b_ten.clone());
     if close_result.is_err() {
         println!("{:?}", close_result);
@@ -73,7 +73,7 @@ pub fn opt_eval_each_subcircuit_until_fail(circuit: CircuitRc, settings: Optimiz
     let topo = toposort_circuit(circuit.clone());
     for circ in topo {
         if circ.info().numel() < max_numel {
-            let reference = evaluate_fn(circ.clone()).unwrap();
+            let reference = evaluate(circ.clone()).unwrap();
             let optimized = optimize_and_evaluate(circ.clone(), settings);
             if let Ok(optimized) = optimized {
                 let close_result = is_close(reference, optimized);
@@ -104,7 +104,7 @@ pub fn randn_ablate_great_grandchildren(circ: CircuitRc) -> CircuitRc {
         .flat_map(|grandchild| grandchild.children().collect::<Vec<_>>());
     let mut replacements: HashMap<HashBytes, CircuitRc> = HashMap::default();
     for great_grandchild in great_grandchildren {
-        let ablated = Circuit::ArrayConstant(ArrayConstant::randn_full(
+        let ablated = Circuit::Array(Array::randn_full(
             great_grandchild.info().shape.clone(),
             great_grandchild.name_cloned(),
             great_grandchild.info().device_dtype.clone(),
