@@ -442,7 +442,7 @@ impl SimpFnSubset {
 #[base_exception(PyValueError)]
 #[derive(Error, Debug, Clone)]
 pub enum SimpConfigError {
-    #[error("names={names:?}")]
+    #[error("names={names:?} ({e_name})")]
     FnNamesNotValid { names: Vec<String> },
 }
 
