@@ -17,7 +17,7 @@ pub fn elim_empty_module(circuit: &Module) -> Option<CircuitRc> {
 #[pyfunction(
     prefix_to_strip = "None",
     module_name = "None",
-    require_all_args = "true",
+    check_all_args_present = "true",
     check_unique_arg_names = "true"
 )]
 pub fn extract_rewrite_raw(
@@ -25,13 +25,13 @@ pub fn extract_rewrite_raw(
     input_specs: Vec<(CircuitRc, ModuleArgSpec)>,
     prefix_to_strip: Option<String>,
     module_name: Option<String>,
-    require_all_args: bool,
+    check_all_args_present: bool,
     check_unique_arg_names: bool,
 ) -> Result<Module> {
     let mut spec = ModuleSpec::new_extract(
         circuit,
         input_specs.clone(),
-        require_all_args,
+        check_all_args_present,
         check_unique_arg_names,
     )?;
     if let Some(pref) = &prefix_to_strip {
