@@ -1,5 +1,5 @@
 use std::{
-    collections::HashSet,
+    collections::BTreeSet,
     fmt::{self, Debug},
     ops,
     sync::Arc,
@@ -29,8 +29,8 @@ use uuid::uuid;
 
 use crate::{
     library::{apply_in_traversal, replace_outside_traversal_symbols},
-    BoundGetter, BoundUpdater, Getter, Matcher, MatcherData, MatcherFromPyBase, MatcherRc,
-    Transform, TransformRc, Updater,
+    AnyFound, BoundAnyFound, BoundGetter, BoundUpdater, Getter, Matcher, MatcherData,
+    MatcherFromPyBase, MatcherRc, TransformRc, Updater,
 };
 
 #[derive(Clone, FromPyObject)]
@@ -49,8 +49,8 @@ pub enum IterativeMatcherData {
     Filter(FilterIterativeMatcher),
     Children(ChildrenMatcher),
     SpecCircuit(IterativeMatcherRc),
-    FilterModuleSpec(IterativeMatcherRc),
-    Chains(HashSet<ChainItem>),
+    NoModuleSpec(IterativeMatcherRc),
+    Chains(BTreeSet<ChainItem>),
     Raw(RawIterativeMatcher),
     PyFunc(PyObject),
 }
@@ -94,8 +94,8 @@ impl From<IterativeMatcherFromPy> for IterativeMatcher {
 
 #[derive(Clone)]
 pub struct ChainItem {
-    first: IterativeMatcherRc,
-    rest: Vec<IterativeMatcherRc>,
+    pub first: IterativeMatcherRc,
+    pub rest: Vec<IterativeMatcherRc>,
     hash: HashBytes,
 }
 
@@ -154,7 +154,7 @@ impl IterativeMatcherData {
             Self::Filter(_) => uuid!("bcc5ccaa-afbe-414d-88b4-b8eac8c93ece"),
             Self::Children(_) => uuid!("0081e9c9-7fdb-43e1-900e-f1adde28f23d"),
             Self::SpecCircuit(_) => uuid!("b728a8c3-0493-4ee5-96c2-71e4900fc0f8"),
-            Self::FilterModuleSpec(_) => uuid!("b728a8c3-0493-4ee5-96c2-71e4900fc0f8"),
+            Self::NoModuleSpec(_) => uuid!("b728a8c3-0493-4ee5-96c2-71e4900fc0f8"),
             Self::Chains(_) => uuid!("958d03ed-7a1a-4ea9-8dd4-d4a8a68feecb"),
             Self::Raw(_) => uuid!("5838ac96-0f48-4cdb-874f-d4f68ce3a52b"),
             Self::PyFunc(_) => uuid!("d3afc3c0-5c86-46df-9e41-7944caedd901"),
@@ -193,7 +193,7 @@ impl IterativeMatcherData {
                     hasher.update(&num.to_le_bytes());
                 }
             }
-            Self::SpecCircuit(matcher) | Self::FilterModuleSpec(matcher) => {
+            Self::SpecCircuit(matcher) | Self::NoModuleSpec(matcher) => {
                 hasher.update(&matcher.hash);
             }
             Self::Chains(chains) => {
@@ -226,7 +226,7 @@ pub struct FilterIterativeMatcher {
     pub end_depth: Option<u32>,
     /// terminate iterative matching if we reach a node which matches this
     pub term_early_at: MatcherRc,
-    depth: u32,
+    pub depth: u32,
 }
 
 impl FilterIterativeMatcher {
@@ -355,13 +355,13 @@ impl FilterIterativeMatcher {
 #[derive(Clone, Debug)]
 pub struct ChildrenMatcher {
     pub iterative_matcher: IterativeMatcherRc,
-    pub child_numbers: HashSet<usize>,
+    pub child_numbers: BTreeSet<usize>,
 }
 
 impl ChildrenMatcher {
     pub fn new(
         iterative_matcher: IterativeMatcherRc,
-        child_numbers: HashSet<usize>,
+        child_numbers: BTreeSet<usize>,
     ) -> Result<Self> {
         Ok(Self {
             iterative_matcher,
@@ -565,14 +565,14 @@ impl IterativeMatcher {
     }
 
     // TODO: this is kinda shitty, see https://github.com/redwoodresearch/unity/issues/1524
-    pub fn validate_matched(&self, matched: &HashSet<CircuitRc>) -> Result<()> {
+    pub fn validate_matched(&self, matched: &BTreeSet<CircuitRc>) -> Result<()> {
         match &self.data {
             IterativeMatcherData::Match(m) => m.validate_matched(matched),
             IterativeMatcherData::Term(_) => Ok(()),
             IterativeMatcherData::Filter(m) => m.iterative_matcher.validate_matched(matched),
             IterativeMatcherData::Children(_) => Ok(()), /* not sure what else you could do here */
             IterativeMatcherData::SpecCircuit(_) => Ok(()), /* not sure what else you could do here */
-            IterativeMatcherData::FilterModuleSpec(m) => m.validate_matched(matched),
+            IterativeMatcherData::NoModuleSpec(m) => m.validate_matched(matched),
             IterativeMatcherData::Chains(m) => {
                 for chain in m {
                     chain.last().validate_matched(matched)?;
@@ -668,7 +668,7 @@ impl IterativeMatcher {
                     }
                 }
             }
-            IterativeMatcherData::FilterModuleSpec(matcher) => {
+            IterativeMatcherData::NoModuleSpec(matcher) => {
                 let is_module = circuit.is_module();
                 let IterateMatchResults { updated, found } = matcher.match_iterate(circuit)?;
                 if is_module {
@@ -679,7 +679,7 @@ impl IterativeMatcher {
                         .map(|(i, x)| {
                             x.and_then(|x| {
                                 (i != 0 && (i - 1) % 2 != 0)
-                                    .then(|| IterativeMatcherData::FilterModuleSpec(x).into())
+                                    .then(|| IterativeMatcherData::NoModuleSpec(x).into())
                             })
                             .into()
                         })
@@ -693,7 +693,7 @@ impl IterativeMatcher {
                 } else {
                     IterateMatchResults {
                         updated: map_updated(updated, |x| {
-                            IterativeMatcherData::FilterModuleSpec(x).into()
+                            IterativeMatcherData::NoModuleSpec(x).into()
                         }),
                         found,
                     }
@@ -742,7 +742,7 @@ impl IterativeMatcher {
                 fn run_item(
                     chain: MaybeChain<'_>,
                     circuit: &CircuitRc,
-                    new_items: &mut HashSet<ChainItemPerChild>,
+                    new_items: &mut BTreeSet<ChainItemPerChild>,
                 ) -> Result<bool> {
                     let IterateMatchResults { updated, found } =
                         chain.first().match_iterate(circuit.clone())?;
@@ -781,7 +781,7 @@ impl IterativeMatcher {
                     }
                 }
 
-                let mut new_items = HashSet::new();
+                let mut new_items: BTreeSet<ChainItemPerChild> = Default::default();
                 let mut any_chain_finished = false;
                 for chain in chains {
                     let new_any_finished =
@@ -797,7 +797,7 @@ impl IterativeMatcher {
                 {
                     assert!(!finished);
                     let num_children = circuit.children().count();
-                    let mut out = vec![HashSet::default(); num_children];
+                    let mut out = vec![BTreeSet::default(); num_children];
                     for item in new_items {
                         match item {
                             ChainItemPerChild::Single(x) => {
@@ -867,7 +867,7 @@ impl IterativeMatcher {
     }
 
     #[pyo3(name = "validate_matched")]
-    fn validate_matched_py(&self, matched: HashSet<CircuitRc>) -> Result<()> {
+    fn validate_matched_py(&self, matched: BTreeSet<CircuitRc>) -> Result<()> {
         self.validate_matched(&matched)
     }
 
@@ -907,7 +907,7 @@ impl IterativeMatcher {
     #[staticmethod]
     pub fn new_children_matcher(
         first_match: IterativeMatcherRc,
-        child_numbers: HashSet<usize>,
+        child_numbers: BTreeSet<usize>,
     ) -> Self {
         first_match.children_matcher(child_numbers)
     }
@@ -1036,7 +1036,7 @@ impl IterativeMatcher {
         IterativeMatcherData::Chains(
             [ChainItem::new(self_.clone().into(), rest)]
                 .into_iter()
-                .collect::<HashSet<_>>(),
+                .collect::<BTreeSet<_>>(),
         )
         .into()
     }
@@ -1052,7 +1052,7 @@ impl IterativeMatcher {
         .into()
     }
 
-    pub fn children_matcher(&self, child_numbers: HashSet<usize>) -> IterativeMatcher {
+    pub fn children_matcher(&self, child_numbers: BTreeSet<usize>) -> IterativeMatcher {
         IterativeMatcherData::Children(ChildrenMatcher {
             iterative_matcher: self_.clone().rc(),
             child_numbers,
@@ -1067,14 +1067,14 @@ impl IterativeMatcher {
     #[args(enable = "true")]
     pub fn filter_module_spec(&self, enable: bool) -> IterativeMatcher {
         if enable {
-            IterativeMatcherData::FilterModuleSpec(self_.clone().rc()).into()
+            IterativeMatcherData::NoModuleSpec(self_.clone().rc()).into()
         } else {
             self_.clone()
         }
     }
 
     #[args(fancy_validate = "Getter::default().default_fancy_validate")]
-    pub fn get(&self, circuit: CircuitRc, fancy_validate: bool) -> Result<HashSet<CircuitRc>> {
+    pub fn get(&self, circuit: CircuitRc, fancy_validate: bool) -> Result<BTreeSet<CircuitRc>> {
         Getter::default().get(circuit, self_.crc(), Some(fancy_validate))
     }
 
@@ -1101,10 +1101,15 @@ impl IterativeMatcher {
         Getter::new(default_fancy_validate).bind(self_.crc())
     }
 
+    pub fn are_any_found(&self, circuit: CircuitRc) -> Result<bool> {
+        AnyFound::new().are_any_found(circuit, self_.crc())
+    }
+    pub fn any_found(&self) -> BoundAnyFound {
+        AnyFound::new().bind(self_.crc())
+    }
+
     #[args(
-        transform_along_modified_path = "Transform::ident().into()",
         cache_transform = "Updater::default().cache_transform",
-        cache_transform_along_modified_path = "Updater::default().cache_transform_along_modified_path",
         cache_update = "Updater::default().cache_update",
         fancy_validate = "Updater::default().default_fancy_validate"
     )]
@@ -1112,44 +1117,32 @@ impl IterativeMatcher {
         &self,
         circuit: CircuitRc,
         transform: TransformRc,
-        transform_along_modified_path: TransformRc,
         cache_transform: bool,
-        cache_transform_along_modified_path: bool,
         cache_update: bool,
         fancy_validate: bool,
     ) -> Result<CircuitRc> {
-        Updater::new(
-            transform,
-            transform_along_modified_path,
-            cache_transform,
-            cache_transform_along_modified_path,
-            cache_update,
-            false,
+        Updater::new(transform, cache_transform, cache_update, false).update(
+            circuit,
+            self_.crc(),
+            Some(fancy_validate),
         )
-        .update(circuit, self_.crc(), Some(fancy_validate))
     }
 
     #[args(
-        transform_along_modified_path = "Transform::ident().into()",
         cache_transform = "Updater::default().cache_transform",
-        cache_transform_along_modified_path = "Updater::default().cache_transform_along_modified_path",
         cache_update = "Updater::default().cache_update",
         default_fancy_validate = "Updater::default().default_fancy_validate"
     )]
     pub fn updater(
         &self,
         transform: TransformRc,
-        transform_along_modified_path: TransformRc,
         cache_transform: bool,
-        cache_transform_along_modified_path: bool,
         cache_update: bool,
         default_fancy_validate: bool,
     ) -> BoundUpdater {
         Updater::new(
             transform,
-            transform_along_modified_path,
             cache_transform,
-            cache_transform_along_modified_path,
             cache_update,
             default_fancy_validate,
         )
@@ -1194,7 +1187,7 @@ pub enum IterativeMatcherError {
 
     #[error("some child_numbers={child_numbers:?} >= num_children={num_children} ({e_name})")]
     ChildNumbersOutOfBounds {
-        child_numbers: HashSet<usize>,
+        child_numbers: BTreeSet<usize>,
         num_children: usize,
     },
 }
