@@ -1,10 +1,10 @@
 //! TODO: fuzzy string matching for debugging (maybe???)
 
 use std::{
-    collections::HashSet,
+    collections::BTreeSet,
     fmt::{self, Debug, Display},
     hash,
-    sync::Arc,
+    sync::{Arc, Mutex},
     vec,
 };
 
@@ -13,17 +13,18 @@ use circuit_base::{visit_circuit, CircuitNode, CircuitNodeUnion, CircuitRc, Circ
 use pyo3::{prelude::*, AsPyPointer};
 use regex::{Captures, Regex};
 use rr_util::{
+    eq_by_big_hash::EqByBigHash,
     make_single_many, setup_callable, simple_default, simple_from,
-    util::{arc_unwrap_or_clone, as_sorted, EmptySingleMany as ESM},
+    util::{arc_unwrap_or_clone, EmptySingleMany as ESM},
 };
 use thiserror::Error;
 use uuid::uuid;
 
-use crate::{IterativeMatcher, IterativeMatcherRc};
+use crate::{BoundAnyFound, IterativeMatcher, IterativeMatcherRc};
 
-make_single_many!(TypeTags, CircuitType, HashSet);
-make_single_many!(Strings, String, HashSet);
-make_single_many!(Circuits, CircuitRc, HashSet);
+make_single_many!(TypeTags, CircuitType, BTreeSet);
+make_single_many!(Strings, String, BTreeSet);
+make_single_many!(Circuits, CircuitRc, BTreeSet);
 
 #[derive(Clone, FromPyObject)]
 pub enum MatcherFromPyBase {
@@ -32,6 +33,7 @@ pub enum MatcherFromPyBase {
     Type(TypeTags),
     Regex(RegexWrap),
     EqM(Circuits),
+    AnyFound(BoundAnyFound),
     Matcher(Matcher),
 }
 
@@ -45,10 +47,11 @@ pub enum MatcherFromPy {
 #[derive(Clone, Debug)]
 pub enum MatcherData {
     Always(bool),
-    Name(HashSet<String>),
-    Type(HashSet<CircuitType>),
+    Name(BTreeSet<String>),
+    Type(BTreeSet<CircuitType>),
     Regex(RegexWrap),
-    EqM(HashSet<CircuitRc>),
+    EqM(BTreeSet<CircuitRc>),
+    AnyFound(Arc<Mutex<BoundAnyFound>>), // sad mutex
     Raw(RawMatcher),
     PyFunc(PyObject),
     Not(MatcherRc),
@@ -73,6 +76,7 @@ impl From<MatcherFromPyBase> for Matcher {
             MatcherFromPyBase::Type(x) => MatcherData::Type(x.into_many()),
             MatcherFromPyBase::Regex(x) => MatcherData::Regex(x),
             MatcherFromPyBase::EqM(x) => MatcherData::EqM(x.into_many()),
+            MatcherFromPyBase::AnyFound(x) => MatcherData::AnyFound(Arc::new(Mutex::new(x))),
             MatcherFromPyBase::Matcher(x) => return x,
         }
         .into()
@@ -155,6 +159,7 @@ impl MatcherData {
             Self::Type(_) => uuid!("16afef04-e938-457b-93fc-6e781e97a63d"),
             Self::Regex(_) => uuid!("29859287-5945-4d04-8c01-6974a2bd3a1d"),
             Self::EqM(_) => uuid!("3d9ee0b2-5075-47f3-9bcd-4e77eb14e5f9"),
+            Self::AnyFound(_) => uuid!("9afd5f57-f738-4e47-a109-da7803546a08"),
             Self::Raw(_) => uuid!("eff8833f-b3c6-4842-8f9b-4404f7abc356"),
             Self::PyFunc(_) => uuid!("f5934590-a3c1-471d-a0a3-1dece4344326"),
             Self::Not(_) => uuid!("b4e14744-4a07-40e4-acd4-a0001e8ffbb0"),
@@ -170,14 +175,14 @@ impl MatcherData {
                 hasher.update(&[*x as u8]);
             }
             Self::Name(x) => {
-                for s in as_sorted(x) {
+                for s in x {
                     hasher.update(s.as_bytes());
                     // variable size so we need to delimit
                     hasher.update(&uuid!("a4b56c19-c5d2-41c2-be29-1742955c1299").into_bytes());
                 }
             }
             Self::Type(x) => {
-                for t in as_sorted(x) {
+                for t in x {
                     hasher.update(&(*t as u32).to_le_bytes());
                 }
             }
@@ -186,10 +191,13 @@ impl MatcherData {
                 hasher.update(x.pattern.as_bytes());
             }
             Self::EqM(x) => {
-                for t in as_sorted(x) {
+                for t in x {
                     hasher.update(&t.info().hash);
                 }
             }
+            Self::AnyFound(x) => {
+                hasher.update(&x.lock().unwrap().matcher.hash());
+            }
             Self::Raw(x) => {
                 hasher.update(&(Arc::as_ptr(&x.0) as *const () as usize).to_le_bytes());
             }
@@ -200,7 +208,7 @@ impl MatcherData {
                 hasher.update(&x.hash);
             }
             Self::Any(x) | Self::All(x) => {
-                for sub in as_sorted(x) {
+                for sub in x {
                     hasher.update(&sub.hash);
                 }
             }
@@ -222,13 +230,13 @@ impl Matcher {
     }
     // TODO: more rust niceness funcs like the py ones!
 
-    pub fn validate_matched(&self, matched: &HashSet<CircuitRc>) -> Result<()> {
-        fn run_on_set<T: Eq + hash::Hash + Debug + Clone, D: Display>(
-            matcher_set: &HashSet<T>,
-            found_set: &HashSet<T>,
+    pub fn validate_matched(&self, matched: &BTreeSet<CircuitRc>) -> Result<()> {
+        fn run_on_set<T: Eq + hash::Hash + Ord + Debug + Clone, D: Display>(
+            matcher_set: &BTreeSet<T>,
+            found_set: &BTreeSet<T>,
             items: &str,
             matcher_type: &str,
-            convert: impl Fn(&HashSet<T>) -> D,
+            convert: impl Fn(&BTreeSet<T>) -> D,
         ) -> Result<()> {
             if !matcher_set.is_subset(found_set) {
                 bail!(
@@ -279,7 +287,8 @@ impl Matcher {
                         + "]"
                 },
             ),
-            MatcherData::Not(_) => Ok(()), // nothing we can check here!
+            MatcherData::AnyFound(_) => Ok(()), // TODO
+            MatcherData::Not(_) => Ok(()),      // nothing we can check here!
             MatcherData::Any(matchers) | MatcherData::All(matchers) => {
                 // TODO: improve errors here?
                 for m in matchers {
@@ -327,6 +336,7 @@ impl Matcher {
             MatcherData::Type(type_tags) => type_tags.contains(&circuit.type_tag()),
             MatcherData::Regex(r) => circuit.name().map(|x| r.call(x)).unwrap_or(false),
             MatcherData::EqM(circs) => circs.contains(&circuit),
+            MatcherData::AnyFound(any_found) => any_found.lock().unwrap().are_any_found(circuit)?,
             MatcherData::Raw(f) => f.0(circuit)?,
             MatcherData::PyFunc(pyfunc) => {
                 Python::with_gil(|py| pyfunc.call1(py, (circuit,)).and_then(|r| r.extract(py)))?
@@ -356,7 +366,7 @@ impl Matcher {
     // TODO: write flatten/simplify method if we want the extra speed + niceness!
 
     #[pyo3(name = "validate_matched")]
-    fn validate_matched_py(&self, matched: HashSet<CircuitRc>) -> Result<()> {
+    fn validate_matched_py(&self, matched: BTreeSet<CircuitRc>) -> Result<()> {
         self.validate_matched(&matched)
     }
 
@@ -400,16 +410,35 @@ impl Matcher {
         Ok(MatcherData::Regex(RegexWrap::new(pattern, escape_dot)?).into())
     }
 
+    #[staticmethod]
+    pub fn match_any_found(finder: IterativeMatcherRc) -> Self {
+        MatcherData::AnyFound(Arc::new(Mutex::new(finder.any_found()))).into()
+    }
+
+    #[staticmethod]
+    pub fn match_any_child_found(finder: IterativeMatcherRc) -> Self {
+        Self::match_any_found(
+            finder
+                .filter(
+                    false,
+                    Some(1),
+                    None,
+                    MatcherFromPyBase::Always(false).into(),
+                )
+                .into(),
+        )
+    }
+
     #[staticmethod]
     #[args(types = "*")]
     pub fn types(types: Vec<CircuitType>) -> Self {
-        MatcherData::Type(types.into_iter().collect::<HashSet<_>>()).into()
+        MatcherData::Type(types.into_iter().collect::<BTreeSet<_>>()).into()
     }
 
     #[staticmethod]
     #[args(circuits = "*")]
     pub fn circuits(circuits: Vec<CircuitRc>) -> Self {
-        MatcherData::EqM(circuits.into_iter().collect::<HashSet<_>>()).into()
+        MatcherData::EqM(circuits.into_iter().collect::<BTreeSet<_>>()).into()
     }
 
     #[staticmethod]
@@ -455,7 +484,6 @@ impl Matcher {
         arc_unwrap_or_clone(other.0).or(self.crc())
     }
 
-    // methods which go via iterative approach
     pub fn to_iterative_matcher(&self) -> IterativeMatcher {
         self.clone().into()
     }
