@@ -1,4 +1,4 @@
-use std::collections::HashSet;
+use std::collections::BTreeSet;
 
 use anyhow::{anyhow, bail, Result};
 use circuit_base::{expand_node::expand_node, CircuitNode, CircuitRc};
@@ -20,17 +20,12 @@ pub struct Updater {
     #[pyo3(get)]
     pub(super) transform: TransformRc,
     #[pyo3(get)]
-    pub(super) transform_along_modified_path: TransformRc,
-    #[pyo3(get)]
     pub(super) cache_transform: bool,
     #[pyo3(get)]
-    pub(super) cache_transform_along_modified_path: bool,
-    #[pyo3(get)]
     pub(super) cache_update: bool,
     #[pyo3(get, set)]
     pub(super) default_fancy_validate: bool,
     pub(super) transform_cache: cached::UnboundCache<HashBytes, CircuitRc>,
-    pub(super) transform_along_modified_path_cache: cached::UnboundCache<HashBytes, CircuitRc>,
     pub(super) updated_cache: cached::UnboundCache<(HashBytes, IterativeMatcherRc), CircuitRc>,
     pub(super) validation_getter: Getter,
 }
@@ -39,13 +34,10 @@ impl Default for Updater {
     fn default() -> Self {
         Self {
             transform: Transform::ident().into(),
-            transform_along_modified_path: Transform::ident().into(),
             cache_transform: true,
-            cache_transform_along_modified_path: true,
             cache_update: true,
             default_fancy_validate: false,
             transform_cache: cached::UnboundCache::new(),
-            transform_along_modified_path_cache: cached::UnboundCache::new(),
             updated_cache: cached::UnboundCache::new(),
             validation_getter: Default::default(),
         }
@@ -56,25 +48,19 @@ impl Default for Updater {
 impl Updater {
     #[new]
     #[args(
-        transform_along_modified_path = "Transform::ident().into()",
         cache_transform = "Updater::default().cache_transform",
-        cache_transform_along_modified_path = "Updater::default().cache_transform_along_modified_path",
         cache_update = "Updater::default().cache_update",
         default_fancy_validate = "Updater::default().default_fancy_validate"
     )]
     pub fn new(
         transform: TransformRc,
-        transform_along_modified_path: TransformRc,
         cache_transform: bool,
-        cache_transform_along_modified_path: bool,
         cache_update: bool,
         default_fancy_validate: bool,
     ) -> Self {
         Self {
             transform,
-            transform_along_modified_path,
             cache_transform,
-            cache_transform_along_modified_path,
             cache_update,
             default_fancy_validate,
             ..Default::default()
@@ -134,12 +120,6 @@ impl Updater {
             if !matches!(self_.transform.data(), TransformData::Ident) {
                 new_circuit = self_.run_transform(new_circuit)?;
             }
-        } else if !matches!(
-            self_.transform_along_modified_path.data(),
-            TransformData::Ident
-        ) && new_circuit != circuit
-        {
-            new_circuit = self_.run_transform_along_modified_path(new_circuit)?;
         }
 
         Ok(new_circuit)
@@ -153,16 +133,6 @@ impl Updater {
     fn run_transform(&mut self, circuit: CircuitRc) -> Result<CircuitRc> {
         self_.transform.run(circuit)
     }
-
-    // macro to dedup as desired
-    #[apply(cached_method)]
-    #[self_id(self_)]
-    #[key(circuit.info().hash)]
-    #[use_try]
-    #[cache_expr(transform_along_modified_path_cache)]
-    fn run_transform_along_modified_path(&mut self, circuit: CircuitRc) -> Result<CircuitRc> {
-        self_.transform_along_modified_path.run(circuit)
-    }
 }
 
 #[pyclass]
@@ -205,7 +175,7 @@ impl BoundUpdater {
 pub struct Getter {
     #[pyo3(get, set)]
     pub(super) default_fancy_validate: bool,
-    pub(super) cache: cached::UnboundCache<(HashBytes, IterativeMatcherRc), HashSet<CircuitRc>>,
+    pub(super) cache: cached::UnboundCache<(HashBytes, IterativeMatcherRc), BTreeSet<CircuitRc>>,
 }
 
 impl Default for Getter {
@@ -234,7 +204,7 @@ impl Getter {
         circuit: CircuitRc,
         matcher: IterativeMatcherRc,
         fancy_validate: Option<bool>,
-    ) -> Result<HashSet<CircuitRc>> {
+    ) -> Result<BTreeSet<CircuitRc>> {
         self.get(circuit, matcher, fancy_validate)
     }
 
@@ -243,7 +213,7 @@ impl Getter {
         circuit: CircuitRc,
         matcher: IterativeMatcherRc,
         fancy_validate: Option<bool>,
-    ) -> Result<HashSet<CircuitRc>> {
+    ) -> Result<BTreeSet<CircuitRc>> {
         let out = self.get_impl(circuit, matcher.clone())?;
         if fancy_validate.unwrap_or(self.default_fancy_validate) {
             matcher.validate_matched(&out)?;
@@ -299,10 +269,10 @@ impl Getter {
         &mut self,
         circuit: CircuitRc,
         matcher: IterativeMatcherRc,
-    ) -> Result<HashSet<CircuitRc>> {
+    ) -> Result<BTreeSet<CircuitRc>> {
         let IterateMatchResults { updated, found } = matcher.match_iterate(circuit.clone())?;
 
-        let mut out = HashSet::new();
+        let mut out: BTreeSet<CircuitRc> = Default::default();
         if found {
             out.insert(circuit.clone());
         }
@@ -342,7 +312,7 @@ impl BoundGetter {
         _py: Python<'_>,
         circuit: CircuitRc,
         fancy_validate: Option<bool>,
-    ) -> Result<HashSet<CircuitRc>> {
+    ) -> Result<BTreeSet<CircuitRc>> {
         self.get(circuit, fancy_validate)
     }
 
@@ -350,7 +320,7 @@ impl BoundGetter {
         &mut self,
         circuit: CircuitRc,
         fancy_validate: Option<bool>,
-    ) -> Result<HashSet<CircuitRc>> {
+    ) -> Result<BTreeSet<CircuitRc>> {
         self.getter
             .get(circuit, self.matcher.clone(), fancy_validate)
     }
@@ -378,6 +348,100 @@ impl BoundGetter {
     }
 }
 
+#[pyclass]
+#[derive(Debug, Clone)]
+pub struct AnyFound {
+    pub(super) cache: cached::UnboundCache<(HashBytes, IterativeMatcherRc), bool>,
+}
+
+#[pymethods]
+impl AnyFound {
+    #[new]
+    pub fn new() -> Self {
+        Self {
+            cache: cached::UnboundCache::new(),
+        }
+    }
+    fn __call__(
+        &mut self,
+        _py: Python<'_>,
+        circuit: CircuitRc,
+        matcher: IterativeMatcherRc,
+    ) -> Result<bool> {
+        self.are_any_found(circuit, matcher)
+    }
+
+    pub fn are_any_found(
+        &mut self,
+        circuit: CircuitRc,
+        matcher: IterativeMatcherRc,
+    ) -> Result<bool> {
+        let out = self.any_impl(circuit, matcher.clone())?;
+        Ok(out)
+    }
+
+    pub fn bind(&self, matcher: IterativeMatcherRc) -> BoundAnyFound {
+        BoundAnyFound {
+            any_found: self.clone(),
+            matcher,
+        }
+    }
+}
+
+impl AnyFound {
+    #[apply(cached_method)]
+    #[self_id(self_)]
+    #[key((circuit.info().hash, matcher.clone()))]
+    #[use_try]
+    #[cache_expr(cache)]
+    fn any_impl(&mut self, circuit: CircuitRc, matcher: IterativeMatcherRc) -> Result<bool> {
+        let IterateMatchResults { updated, found } = matcher.match_iterate(circuit.clone())?;
+        if found {
+            return Ok(true);
+        }
+
+        if !all_finished(&updated) {
+            let new_matchers = per_child(updated, matcher, circuit.num_children());
+            for (child, new_matcher) in circuit.children().zip(new_matchers) {
+                if let Some(new_matcher) = new_matcher {
+                    if self_.any_impl(child, new_matcher)? {
+                        return Ok(true);
+                    }
+                }
+            }
+        }
+        Ok(false)
+    }
+}
+
+#[pyclass]
+#[derive(Debug, Clone)]
+pub struct BoundAnyFound {
+    #[pyo3(get, set)]
+    pub any_found: AnyFound,
+    #[pyo3(get, set)]
+    pub matcher: IterativeMatcherRc,
+}
+
+#[pymethods]
+impl BoundAnyFound {
+    #[new]
+    pub fn new(any_found: AnyFound, matcher: IterativeMatcherRc) -> Self {
+        Self {
+            any_found,
+            matcher: matcher.into(),
+        }
+    }
+
+    fn __call__(&mut self, _py: Python<'_>, circuit: CircuitRc) -> Result<bool> {
+        self.are_any_found(circuit)
+    }
+
+    pub fn are_any_found(&mut self, circuit: CircuitRc) -> Result<bool> {
+        self.any_found.are_any_found(circuit, self.matcher.clone())
+    }
+}
+
 #[pyclass]
 #[derive(Debug, Clone)]
 pub struct Expander {
