@@ -61,10 +61,10 @@ pub fn diff_circuits(
         hash_child_count: bool,
         hash_child_shapes: bool,
     ) -> Result<()> {
-        const SAME_SELF_COLOR: usize = 4;
-        const SAME_COLOR: usize = 0;
-        const NEW_COLOR: usize = 2;
-        const REMOVED_COLOR: usize = 1;
+        const SAME_SELF_COLOR: usize = 3;
+        const SAME_COLOR: usize = 4; // TODO: this is just a random color atm
+        const NEW_COLOR: usize = 1;
+        const REMOVED_COLOR: usize = 0;
         if new == old {
             let mut new_options = options.clone();
             new_options.colorer = Some(PrintOptions::fixed_color(SAME_COLOR));
