@@ -1,7 +1,9 @@
-use anyhow::Result;
+use anyhow::{Context, Result};
 use circuit_base::{prelude::*, CircuitType, Index};
 use circuit_rewrites::algebraic_rewrite::{index_elim_identity, push_down_index_raw};
-use get_update_node::{IterativeMatcher, IterativeMatcherRc, Matcher};
+use get_update_node::{
+    iterative_matcher::require_single, IterativeMatcher, IterativeMatcherRc, Matcher,
+};
 use macro_rules_attribute::apply;
 use pyo3::prelude::*;
 use rr_util::{cached_lambda, util::HashBytes};
@@ -40,21 +42,23 @@ pub fn push_down_index(
                 return Ok(removed_node);
             }
         }
-        let new_matcher = if let (Some(new_matcher), _) = matcher
+        let updated = matcher
             .match_iterate(index.node.clone())?
-            .none_if_finished(matcher)
+            .unwrap_or_same(matcher)
+            .0;
+        if let Some(new_matcher) = require_single(updated)
+            .context("push down index doesn't support matcher per child")?
+            .0
         {
-            new_matcher
+            push_down_index_raw(
+                &index,
+                allow_partial_pushdown,
+                |new_index| push_down_rec(new_index, new_matcher.clone()),
+                suffix.clone(),
+            )
         } else {
-            return Ok(index.rc());
-        };
-
-        push_down_index_raw(
-            &index,
-            allow_partial_pushdown,
-            |new_index| push_down_rec(new_index, new_matcher.clone()),
-            suffix.clone(),
-        )
+            Ok(index.rc())
+        }
     }
 
     push_down_rec(index, traversal)
