@@ -5,14 +5,16 @@ use circuit_base::{computational_node::EinsumArgs, Add, Circuit, CircuitNode, Ci
 use circuit_rewrites::algebraic_rewrite::{
     add_flatten_once, make_einsum_ints_same_one_layer_and_int_info,
 };
-use get_update_node::{IterativeMatcher, IterativeMatcherRc, MatcherData};
+use get_update_node::{
+    iterative_matcher::UpdatedIterativeMatcher, IterativeMatcher, IterativeMatcherRc,
+};
 use macro_rules_attribute::apply;
 use pyo3::{exceptions::PyValueError, prelude::*};
 use rr_util::{
     python_error_exception,
     rearrange_spec::{check_permutation, PermError},
     union_find::UnionFind,
-    util::{is_unique, AsOp, AxisInt, EinsumAxes, Multizip},
+    util::{is_unique, transpose, AsOp, AxisInt, EinsumAxes, Multizip},
 };
 use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
 use thiserror::Error;
@@ -999,17 +1001,18 @@ pub fn finish_with_extra_and_circ<I: NestFullInfo>(
 pub fn run_matcher<T>(
     circ: &CircuitRc,
     matcher: IterativeMatcherRc,
-) -> Result<Option<(&T, IterativeMatcherRc)>>
+) -> Result<Option<(&T, UpdatedIterativeMatcher)>>
 where
     Circuit: AsOp<T>,
 {
     if let Some(out) = AsOp::<T>::as_op(&***circ) {
         // if let chain ICE's compiler : / (fixed on new version)
-        if let (Some(new_matcher), _) = matcher
+        let updated = matcher
             .match_iterate(circ.clone())?
-            .none_if_finished(matcher)
-        {
-            return Ok(Some((out, new_matcher)));
+            .unwrap_or_same(matcher)
+            .0;
+        if !updated.all_finished() {
+            return Ok(Some((out, updated)));
         }
     }
     Ok(None)
@@ -1025,37 +1028,36 @@ pub fn flatten_impl<Full: NestFullInfo>(
 where
     Circuit: AsOp<Full::CircuitType>,
 {
+    let num_children = circ.num_children();
     let (updated_current_matcher_to_pair, found): (Vec<_>, Vec<_>) = current_matcher_to_pair
         .into_iter()
         .map(|m| {
             m.match_iterate(circ.clone()).map(|out| {
-                let (new, found) = out.none_if_finished(m);
-                (
-                    // if we've finished, never match anything else (keep placeholder for index)
-                    new.unwrap_or(MatcherData::Always(false).into()),
-                    found,
-                )
+                let (new, found) = out.unwrap_or_same(m);
+                (new.per_child_with_term(num_children), found)
             })
         })
         .collect::<Result<Vec<_>>>()?
         .into_iter()
         .unzip();
+    let updated_current_matcher_to_pair = transpose(updated_current_matcher_to_pair, num_children);
 
     // TODO: we could cache, but probably overkill
     // no if let chain is sad, but format is broke.
     let f = || -> Result<_> {
         if let Some(out) = AsOp::<Full::CircuitType>::as_op(&**circ) {
-            if let (Some(new_matcher), _) = matcher
+            let updated = matcher
                 .match_iterate(circ.clone())?
-                .none_if_finished(matcher)
-            {
-                return Ok(Some((out, new_matcher)));
+                .unwrap_or_same(matcher)
+                .0;
+            if !updated.all_finished() {
+                return Ok(Some((out, updated)));
             }
         }
         Ok(None)
     };
 
-    let (x, new_matcher) = if let Some(x) = f()? {
+    let (x, updated) = if let Some(x) = f()? {
         x
     } else {
         return Ok((
@@ -1070,12 +1072,10 @@ where
 
     let rec_on_children = x
         .children()
-        .map(|c| {
-            flatten_impl(
-                c,
-                new_matcher.clone(),
-                updated_current_matcher_to_pair.clone(),
-            )
+        .zip(updated.per_child_with_term(num_children))
+        .zip(updated_current_matcher_to_pair)
+        .map(|((c, new_matcher), updated_matchers_to_pair)| {
+            flatten_impl(c, new_matcher.clone(), updated_matchers_to_pair)
         })
         .collect::<Result<Vec<_>>>()?;
     let x = x.map_children_unwrap_idxs(|i| rec_on_children[i].0.clone());
@@ -1256,9 +1256,9 @@ pub fn register(py: Python<'_>, m: &PyModule) -> PyResult<()> {
 #[base_exception(PyValueError)]
 #[derive(Error, Debug, Clone)]
 pub enum NestError {
-    #[error("count_rest={count_rest}")]
+    #[error("count_rest={count_rest} ({e_name})")]
     MultipleRest { count_rest: usize },
-    #[error("matcher={matcher:?} {}={other_matcher:?}\n{ints:?} intersects with {other_ints:?}\nintersection={intersection:?}\nintersection_circs={intersection_circs:?}", 
+    #[error("matcher={matcher:?} {}={other_matcher:?}\n{ints:?} intersects with {other_ints:?}\nintersection={intersection:?}\nintersection_circs={intersection_circs:?} ({e_name})", 
         "overlaps with other_matcher")]
     MatchersOverlap {
         matcher: IterativeMatcherRc,
@@ -1269,7 +1269,7 @@ pub enum NestError {
         intersection_circs: Vec<CircuitRc>,
     },
     #[error(
-        "matcher={matcher:?}, matcher_ints={matcher_ints:?} {}{explicit_ints:?}\nintersection={intersection:?}\n{}{intersection_circs:?}", 
+        "matcher={matcher:?}, matcher_ints={matcher_ints:?} {}{explicit_ints:?}\nintersection={intersection:?}\n{}{intersection_circs:?} ({e_name})", 
         "intersects with explicit_ints=", 
         "intersection_circs=")]
     MatcherOverlapsWithExplicitInts {
@@ -1279,32 +1279,32 @@ pub enum NestError {
         intersection: Vec<usize>,
         intersection_circs: Vec<CircuitRc>,
     },
-    #[error("matcher={matcher:?} matches={matches:?} (int_matches={int_matches:?})")]
+    #[error("matcher={matcher:?} matches={matches:?} (int_matches={int_matches:?}) ({e_name})")]
     MatcherMatchedMultipleAndMustBeUnique {
         matcher: IterativeMatcherRc,
         matches: Vec<CircuitRc>,
         int_matches: Vec<usize>,
     },
-    #[error("matcher={matcher:?}")]
+    #[error("matcher={matcher:?} ({e_name})")]
     MatcherMatchedNoneAndMustExist { matcher: IterativeMatcherRc },
-    #[error("traversal={traversal:?} circuit={circuit:?}")]
+    #[error("traversal={traversal:?} circuit={circuit:?} ({e_name})")]
     TraversalMatchedNothing {
         traversal: IterativeMatcherRc,
         circuit: CircuitRc,
     },
-    #[error("ints={ints:?} count={count} extra_ints={extra_ints:?}")]
+    #[error("ints={ints:?} count={count} extra_ints={extra_ints:?} ({e_name})")]
     IntNotContainedInRangeCount {
         ints: Vec<usize>,
         count: usize,
         extra_ints: HashSet<usize>,
     },
-    #[error("perm={perm:?} expected_len={expected_len}")]
+    #[error("perm={perm:?} expected_len={expected_len} ({e_name})")]
     PermHasWrongLen {
         perm: EinsumAxes,
         expected_len: usize,
     },
-    #[error("This num wasn't present in orig!")]
+    #[error("This num wasn't present in orig! ({e_name})")]
     OrigNumPermWhenNotPresentInOrig {},
-    #[error("missed_idxs={missed_idxs:?}")]
+    #[error("missed_idxs={missed_idxs:?} ({e_name})")]
     PermutationMissesIdxsAndNoRestInSpec { missed_idxs: HashSet<usize> },
 }
