@@ -1,9 +1,11 @@
-use std::{cmp::min, fs, io::Write, iter::zip, os::unix::prelude::OsStrExt};
+use std::{cmp::min, fs, io::Write, iter::zip, os::unix::prelude::OsStrExt, sync::mpsc::channel};
 
 use anyhow::{anyhow, bail, Context, Result};
 use base16::encode_lower;
+use itertools::Itertools;
 use pyo3::prelude::*;
 use rustc_hash::FxHashSet as HashSet;
+use threadpool::ThreadPool;
 
 use crate::{
     py_types::{Tensor, PY_UTILS},
@@ -161,24 +163,54 @@ pub fn save_tensor(tensor: Tensor, force: bool) -> Result<bool> {
     result
 }
 
-#[pyfunction]
-pub fn sync_all_unsynced_tensors() -> Result<()> {
+fn copy_without_setting_permissions(local_filename: &str, remote_filename: &str) -> Result<()> {
+    let mut reader = fs::File::open(local_filename)
+        .context(format!("opening {} in read mode failed", local_filename))?;
+    let mut writer = fs::File::create(remote_filename)
+        .context(format!("opening {} in write mode failed", remote_filename))?;
+    std::io::copy(&mut reader, &mut writer).context("writing contents failed")?;
+    Ok(())
+}
+
+#[pyfunction(parallelism = "15")]
+pub fn sync_all_unsynced_tensors(parallelism: usize) -> Result<()> {
     let list_dir = get_tensor_cache_dir() + "/.unsynced_list";
     println!("starting to read list");
     let all_unsynced = fs::read_to_string(&list_dir)?;
-    println!("read list");
-    let lines = all_unsynced[..all_unsynced.len() - 1].lines();
-    for line in lines {
-        let local_filename = get_filename_dir_tree(&get_tensor_cache_dir(), line)?;
-        println!("have dir tree");
-        let remote_filename =
-            local_filename.replace(&get_tensor_cache_dir(), &get_rrfs_tensor_db_dir());
-        let remote_filename_dir: Vec<_> = remote_filename.split("/").collect();
-        let remote_filename_dir = remote_filename_dir[..remote_filename_dir.len() - 1].join("/");
-        fs::create_dir_all(&remote_filename_dir).context("failed to create dir in tree")?;
-        fs::copy(local_filename, remote_filename).context("failed to copy")?;
-        print!("wrote");
+    println!("read list {}", all_unsynced.len());
+    let lines: Vec<String> = all_unsynced
+        .lines()
+        .filter(|x| !x.is_empty())
+        .map(|x| x.to_owned())
+        .collect();
+    // we can have more threads than cores bc we're not actually doing computation on threads
+    // ideally would be using async
+    let n_workers = parallelism;
+    let pool = ThreadPool::new(n_workers);
+    let (tx, rx) = channel();
+    for line in &lines {
+        let line = line.clone();
+        let tx = tx.clone();
+        pool.execute(move || {
+            tx.send((move || {
+                let local_filename = get_filename_dir_tree(&get_tensor_cache_dir(), &line)?;
+                println!("have dir tree");
+                let remote_filename =
+                    local_filename.replace(&get_tensor_cache_dir(), &get_rrfs_tensor_db_dir());
+                let remote_filename_dir: Vec<_> = remote_filename.split("/").collect();
+                let remote_filename_dir = remote_filename_dir
+                    [..remote_filename_dir.len().checked_add_signed(-1).unwrap()]
+                    .join("/");
+                fs::create_dir_all(&remote_filename_dir).context("failed to create dir in tree")?;
+                copy_without_setting_permissions(&local_filename, &remote_filename)
+                    .context("failed to copy")?;
+                print!("wrote");
+                Ok::<(), anyhow::Error>(())
+            })())
+            .unwrap()
+        });
     }
+    rx.iter().take(lines.len()).collect::<Result<Vec<()>>>()?;
     fs::write(&list_dir, &fs::read(&list_dir)?[all_unsynced.len()..]).context("write failed")
 }
 
