@@ -2,8 +2,8 @@ use std::{
     cmp::{max, min},
     fmt::{self, Display},
     iter::zip,
-    num::{ParseFloatError, ParseIntError},
     ops::Deref,
+    str::FromStr,
 };
 
 use anyhow::{bail, Context, Error, Result};
@@ -678,7 +678,7 @@ impl TensorIndex {
         string: &str,
         tensor_cache: &mut Option<TensorCacheRrfs>,
     ) -> Result<Self> {
-        let er = ParseError::Fail {
+        let er = ParseError::InvalidIndex {
             string: string.to_owned(),
         };
         Ok(TensorIndex(
@@ -691,8 +691,8 @@ impl TensorIndex {
                 .map(str::trim)
                 .filter(|x| !x.is_empty())
                 .map(|x| -> Result<_> {
-                    // TODO: fix error handle + trim whitespace
-                    if let Ok(int) = x.parse::<i64>() {
+                    // TODO: trim whitespace?
+                    if let Ok(int) = parse_numeric(x) {
                         return Ok(TensorAxisIndex::Single(int));
                     }
                     if let Some(no_t) = x.strip_prefix('t') {
@@ -714,8 +714,7 @@ impl TensorIndex {
                         if item.is_empty() {
                             Ok(None)
                         } else {
-                            item.parse::<i64>()
-                                .map_err(ParseError::from)
+                            parse_numeric(item)
                                 .with_context(|| {
                                     format!(
                                         "failed to parse {} of slice, index string='{}'",
@@ -980,130 +979,30 @@ impl Default for TorchDeviceDtype {
     }
 }
 
-const REGEX_101_MSG: &str = "\nTry using regex101.com : )\n(TODO: auto generate regex test page via https://github.com/firasdib/Regex101/wiki/API#python-3)";
-
 // TODO: break up and move part to parsing.rs
 #[apply(python_error_exception)]
 #[base_error_name(Parse)]
 #[base_exception(PyValueError)]
 #[derive(Error, Debug, Clone)]
 pub enum ParseError {
-    #[error(
-        "regex didn't match '{line}'\nregex='{regex_string}'{}\n({e_name})",
-        REGEX_101_MSG
-    )]
-    RegexDidntMatch { line: String, regex_string: String },
-
-    #[error(
-        "regex didn't match group={group} for '{line}'\n{}\nregex='{regex_string}'{}\n({e_name})",
-        "Note: this error can also be caused by failing to lookup a reference circuit",
-        REGEX_101_MSG
-    )]
-    RegexDidntMatchGroup {
-        line: String,
-        regex_string: String,
-        group: usize,
-    },
-
-    #[error("Parsing number failed err={err_string} ({e_name})")]
-    NumberFail { err_string: String },
+    #[error("Parsing number failed string='{string}', err={err_string} ({e_name})")]
+    InvalidNumber { string: String, err_string: String },
 
     #[error("extra='{extra}' ({e_name})")]
     ExpectedNoExtraInfo { extra: String },
 
-    #[error("Parsing failed {string} ({e_name})")]
-    Fail { string: String },
+    #[error("string='{string}'\nindex should look like '[1,3:5,f6d587856c8444a9]'\n({e_name})")]
+    InvalidIndex { string: String },
 
-    #[error("Expected UUID string, found  '{string}'\nerr_msg={err_msg} ({e_name})")]
+    #[error("Expected UUID string, found '{string}'\nerr_msg={err_msg} ({e_name})")]
     InvalidUuid { string: String, err_msg: String },
 
-    #[error("Parsing wrong number of children, expected {expected} found {found}, ident={ident} ({e_name})")]
-    WrongNumberChildren {
-        expected: usize,
-        found: usize,
-        ident: String,
-    },
-
-    #[error("effectively negative indent!\n{}\n{}\n{}\n({e_name})",
-        format!("this_num_spaces_base={} < first_num_spaces={}", this_num_spaces_base, first_num_spaces),
-        format!("first_line='{}'", first_line),
-        format!("this_line='{}'", this_line)
-        )]
-    LessIndentationThanFirstItem {
-        this_num_spaces_base: usize,
-        first_num_spaces: usize,
-        first_line: String,
-        this_line: String,
-    },
-
-    #[error("Parsing invalid indentation, tab width {tab_width} num spaces {spaces} stack indentation {stack_indentation} ({e_name})")]
-    InvalidIndentation {
-        spaces: usize,
-        tab_width: usize,
-        stack_indentation: usize,
-    },
-
-    #[error("Parsing invalid circuit variant {v} ({e_name})")]
-    InvalidVariant { v: String },
-
-    #[error("Parsing invalid serial number {serial_number} ({e_name})")]
-    InvalidSerialNumber { serial_number: usize },
-
-    #[error("Parsing Extra Unneeded String '{string}' ({e_name})")]
-    ExtraUnneededString { string: String },
-
-    #[error("Parsing shape needed but not provided on {variant} ({e_name})")]
-    ShapeNeeded { variant: String },
-
-    #[error("Circuit by name not found: {name} ({e_name})")]
-    CircuitRefNotFound { name: String },
-
     #[error("Einsum string invalid {string} {substring} ({e_name})")]
     EinsumStringInvalid { string: String, substring: String },
 
     #[error("Einsum string doesn't have arrow {string} ({e_name})")]
     EinsumStringNoArrow { string: String },
 
-    #[error("actual_num_circuits={actual_num_circuits} ({e_name})")]
-    ExpectedOneCircuitGotMultiple { actual_num_circuits: usize },
-
-    #[error("expected 't' or 'f' got='{got}' ({e_name})")]
-    InvalidTerseBool { got: String },
-
-    #[error("reference_name={reference_name} ({e_name})")]
-    ReferenceCircuitHasChildrenInParseStr { reference_name: String },
-
-    #[error("reference_name={reference_name} ({e_name})")]
-    ReferenceCircuitNameFollowedByAdditionalInfo {
-        reference_name: String,
-        line: String,
-    },
-
-    #[error("repeat_ident={repeat_ident} \n{err_strs}line='{line}' ({e_name})")]
-    OnCircuitRepeatInfoIsNotSame {
-        repeat_ident: String,
-        line: String,
-        err_strs: String,
-    },
-
-    #[error("num_chars={num_chars} extra='{extra}' name={name:?}\nmodule extra should look like 'tt,tf,ft' and we check for 2 chars per comma delimited item.\n({e_name})")]
-    Expected2CharsPerForModuleExtra {
-        name: Option<String>,
-        num_chars: usize,
-        extra: String,
-    },
-
-    #[error("num_children={num_children} != 2 * num_args={num_args:?} + 1\nname={name:?} extra={extra}\n{}\n{}\n{}\n({e_name})",
-    "modules have 1 child for the spec and then a symbol and a input circuit for each arg (2 * num_args + 1)",
-    "this implies the number of children is always odd",
-    "num_args is based on parsing the batchable and expandable info from 'extra' (see above) (in a form like 'tt,tf,ft' (Some(3)) or 'all_t' (None))")]
-    ModuleInconsistentNumChildren {
-        name: Option<String>,
-        num_children: usize,
-        extra: String,
-        num_args: Option<usize>,
-    },
-
     #[error("factors={factors:?} string='{string}' ({e_name})")]
     FactorProductTooLarge { factors: Vec<usize>, string: String },
 
@@ -1111,18 +1010,43 @@ pub enum ParseError {
     SymbolicSizeNumberOutOfBounds { i: usize, bound: usize },
 }
 
-impl From<ParseIntError> for ParseError {
-    fn from(value: ParseIntError) -> Self {
-        ParseError::NumberFail {
-            err_string: value.to_string(),
-        }
-    }
+pub const UINT_WITH_UNDERSCORE: &str = r"\d[\d_]*";
+
+pub fn parse_numeric<T: FromStr>(x: &str) -> Result<T, ParseError>
+where
+    T::Err: fmt::Display,
+{
+    // this allows (e.g.) stuff like 38473.___3 or -___3 which isn't allowed in rust. Seems fine to me (for now).
+    (x.chars().take(1).collect::<String>()
+        + &x.chars().skip(1).collect::<String>().replace('_', ""))
+        .parse::<T>()
+        .map_err(|e| ParseError::InvalidNumber {
+            err_string: e.to_string(),
+            string: x.to_owned(),
+        })
 }
 
-impl From<ParseFloatError> for ParseError {
-    fn from(value: ParseFloatError) -> Self {
-        ParseError::NumberFail {
-            err_string: value.to_string(),
-        }
-    }
+#[test]
+fn test_parse_numeric() {
+    assert!(parse_numeric::<usize>("hi").is_err());
+    assert!(parse_numeric::<usize>("_1").is_err());
+    assert!(parse_numeric::<usize>("__2").is_err());
+    assert!(parse_numeric::<usize>("_387483741").is_err());
+    assert!(parse_numeric::<usize>("_38748_3741").is_err());
+    assert!(parse_numeric::<f64>("_.38748_3741").is_err());
+    assert!(parse_numeric::<f64>("_1.1").is_err());
+
+    assert_eq!(parse_numeric::<usize>("38748_3741").unwrap(), 38748_3741);
+    assert_eq!(
+        parse_numeric::<usize>("38748_____3741").unwrap(),
+        38748_____3741
+    );
+    assert_eq!(
+        parse_numeric::<usize>("3_____8748_3741").unwrap(),
+        3_____8748_3741
+    );
+    assert_eq!(parse_numeric::<f64>("3.38473").unwrap(), 3.38473);
+    assert_eq!(parse_numeric::<f64>("3.").unwrap(), 3.);
+    assert_eq!(parse_numeric::<f64>("3____.3").unwrap(), 3____.3);
+    assert_eq!(parse_numeric::<f64>("3.____3__").unwrap(), 3____.3__);
 }
