@@ -130,6 +130,8 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
     m.add_class::<get_update_node::BoundUpdater>()?;
     m.add_class::<get_update_node::Getter>()?;
     m.add_class::<get_update_node::BoundGetter>()?;
+    m.add_class::<get_update_node::AnyFound>()?;
+    m.add_class::<get_update_node::BoundAnyFound>()?;
     m.add_class::<get_update_node::Expander>()?;
     m.add_class::<circuit_base::print::PrintOptions>()?;
     m.add_class::<circuit_base::print_html::PrintHtmlOptions>()?;
@@ -386,6 +388,18 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
     m.add_class::<get_update_node::sampler::RandomSampleSpec>()?;
     m.add_class::<get_update_node::sampler::RunDiscreteVarAllSpec>()?;
     m.add_class::<get_update_node::sampler::Sampler>()?;
+    m.add_function(wrap_pyfunction!(
+        get_update_node::matcher_debug::append_matchers_to_names,
+        m
+    )?)?;
+    m.add_function(wrap_pyfunction!(
+        get_update_node::matcher_debug::print_matcher_debug,
+        m
+    )?)?;
+    m.add_function(wrap_pyfunction!(
+        get_update_node::matcher_debug::repr_matcher_debug,
+        m
+    )?)?;
     m.add_function(wrap_pyfunction!(
         get_update_node::sampler::factored_cumulant_expectation_rewrite,
         m
